@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 ##
 # = Trigonometric and transcendental functions for complex numbers.
 #
@@ -7,65 +8,65 @@
 #
 # Note that the selection of functions is similar, but not identical,
 # to that in module math. The reason for having two modules is that
-# some users aren’t interested in complex numbers, and perhaps don’t
+# some users aren't interested in complex numbers, and perhaps don't
 # even know what they are. They would rather have Math.sqrt(-1) raise
 # an exception than return a complex number.
 #
+# For more information you can see Complex class.
+#
 # == Usage
 #
 # To start using this library, simply require cmath library:
 #
 #   require "cmath"
-#
-# And after call any CMath function. For example:
-#
-#   CMath.sqrt(-9)          #=> 0+3.0i
-#   CMath.exp(0 + 0i)       #=> 1.0+0.0i
-#   CMath.log10(-5.to_c)    #=> (0.6989700043360187+1.3643763538418412i)
-#
-#
-# For more information you can see Complec class.
 
 module CMath
 
   include Math
 
-  alias exp! exp
-  alias log! log
-  alias log2! log2
-  alias log10! log10
-  alias sqrt! sqrt
-  alias cbrt! cbrt
+  # Backup of Math is needed because mathn.rb replaces Math with CMath.
+  RealMath = Math # :nodoc:
+  private_constant :RealMath
 
-  alias sin! sin
-  alias cos! cos
-  alias tan! tan
-
-  alias sinh! sinh
-  alias cosh! cosh
-  alias tanh! tanh
-
-  alias asin! asin
-  alias acos! acos
-  alias atan! atan
-  alias atan2! atan2
-
-  alias asinh! asinh
-  alias acosh! acosh
-  alias atanh! atanh
+  %w[
+    exp
+    log
+    log2
+    log10
+    sqrt
+    cbrt
+    sin
+    cos
+    tan
+    sinh
+    cosh
+    tanh
+    asin
+    acos
+    atan
+    atan2
+    asinh
+    acosh
+    atanh
+  ].each do |meth|
+    define_method(meth + '!') do |*args, &block|
+      warn("CMath##{meth}! is deprecated; use CMath##{meth} or Math##{meth}", uplevel: 1) if $VERBOSE
+      RealMath.send(meth, *args, &block)
+    end
+  end
 
   ##
   # Math::E raised to the +z+ power
   #
-  #   CMath.exp(2i) #=> (-0.4161468365471424+0.9092974268256817i)
+  #   CMath.exp(1.i * Math::PI) #=> (-1.0+1.2246467991473532e-16i)
   def exp(z)
     begin
       if z.real?
-	exp!(z)
+        RealMath.exp(z)
       else
-	ere = exp!(z.real)
-	Complex(ere * cos!(z.imag),
-		ere * sin!(z.imag))
+        ere = RealMath.exp(z.real)
+        Complex(ere * RealMath.cos(z.imag),
+                ere * RealMath.sin(z.imag))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -78,20 +79,12 @@ module CMath
   #
   #   CMath.log(1 + 4i)     #=> (1.416606672028108+1.3258176636680326i)
   #   CMath.log(1 + 4i, 10) #=> (0.6152244606891369+0.5757952953408879i)
-  def log(*args)
+  def log(z, b=::Math::E)
     begin
-      z, b = args
-      unless b.nil? || b.kind_of?(Numeric)
-	raise TypeError,  "Numeric Number required"
-      end
-      if z.real? and z >= 0 and (b.nil? or b >= 0)
-	log!(*args)
+      if z.real? && z >= 0 && b >= 0
+        RealMath.log(z, b)
       else
-	a = Complex(log!(z.abs), z.arg)
-	if b
-	  a /= log(b)
-        end
-        a
+        Complex(RealMath.log(z.abs), z.arg) / log(b)
       end
     rescue NoMethodError
       handle_no_method_error
@@ -99,15 +92,15 @@ module CMath
   end
 
   ##
-  # returns the base 2 logarithm of +z+
+  # Returns the base 2 logarithm of +z+
   #
   #   CMath.log2(-1) => (0.0+4.532360141827194i)
   def log2(z)
     begin
       if z.real? and z >= 0
-	log2!(z)
+        RealMath.log2(z)
       else
-	log(z) / log!(2)
+        log(z) / RealMath.log(2)
       end
     rescue NoMethodError
       handle_no_method_error
@@ -115,15 +108,15 @@ module CMath
   end
 
   ##
-  # returns the base 10 logarithm of +z+
+  # Returns the base 10 logarithm of +z+
   #
   #   CMath.log10(-1) #=> (0.0+1.3643763538418412i)
   def log10(z)
     begin
       if z.real? and z >= 0
-	log10!(z)
+        RealMath.log10(z)
       else
-	log(z) / log!(10)
+        log(z) / RealMath.log(10)
       end
     rescue NoMethodError
       handle_no_method_error
@@ -137,20 +130,20 @@ module CMath
   def sqrt(z)
     begin
       if z.real?
-	if z < 0
-	  Complex(0, sqrt!(-z))
-	else
-	  sqrt!(z)
-	end
+        if z < 0
+          Complex(0, RealMath.sqrt(-z))
+        else
+          RealMath.sqrt(z)
+        end
       else
-	if z.imag < 0 ||
-	    (z.imag == 0 && z.imag.to_s[0] == '-')
-	  sqrt(z.conjugate).conjugate
-	else
-	  r = z.abs
-	  x = z.real
-	  Complex(sqrt!((r + x) / 2.0), sqrt!((r - x) / 2.0))
-	end
+        if z.imag < 0 ||
+            (z.imag == 0 && z.imag.to_s[0] == '-')
+          sqrt(z.conjugate).conjugate
+        else
+          r = z.abs
+          x = z.real
+          Complex(RealMath.sqrt((r + x) / 2.0), RealMath.sqrt((r - x) / 2.0))
+        end
       end
     rescue NoMethodError
       handle_no_method_error
@@ -158,7 +151,7 @@ module CMath
   end
 
   ##
-  # returns the principal value of the cube root of +z+
+  # Returns the principal value of the cube root of +z+
   #
   #   CMath.cbrt(1 + 4i) #=> (1.449461632813119+0.6858152562177092i)
   def cbrt(z)
@@ -166,16 +159,16 @@ module CMath
   end
 
   ##
-  # returns the sine of +z+, where +z+ is given in radians
+  # Returns the sine of +z+, where +z+ is given in radians
   #
   #   CMath.sin(1 + 1i) #=> (1.2984575814159773+0.6349639147847361i)
   def sin(z)
     begin
       if z.real?
-	sin!(z)
+        RealMath.sin(z)
       else
-	Complex(sin!(z.real) * cosh!(z.imag),
-		cos!(z.real) * sinh!(z.imag))
+        Complex(RealMath.sin(z.real) * RealMath.cosh(z.imag),
+                RealMath.cos(z.real) * RealMath.sinh(z.imag))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -183,16 +176,16 @@ module CMath
   end
 
   ##
-  # returns the cosine of +z+, where +z+ is given in radians
+  # Returns the cosine of +z+, where +z+ is given in radians
   #
   #   CMath.cos(1 + 1i) #=> (0.8337300251311491-0.9888977057628651i)
   def cos(z)
     begin
       if z.real?
-	cos!(z)
+        RealMath.cos(z)
       else
-	Complex(cos!(z.real) * cosh!(z.imag),
-		-sin!(z.real) * sinh!(z.imag))
+        Complex(RealMath.cos(z.real) * RealMath.cosh(z.imag),
+                -RealMath.sin(z.real) * RealMath.sinh(z.imag))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -200,15 +193,15 @@ module CMath
   end
 
   ##
-  # returns the tangent of +z+, where +z+ is given in radians
+  # Returns the tangent of +z+, where +z+ is given in radians
   #
   #   CMath.tan(1 + 1i) #=> (0.27175258531951174+1.0839233273386943i)
   def tan(z)
     begin
       if z.real?
-	tan!(z)
+        RealMath.tan(z)
       else
-	sin(z) / cos(z)
+        sin(z) / cos(z)
       end
     rescue NoMethodError
       handle_no_method_error
@@ -216,16 +209,16 @@ module CMath
   end
 
   ##
-  # returns the hyperbolic sine of +z+, where +z+ is given in radians
+  # Returns the hyperbolic sine of +z+, where +z+ is given in radians
   #
   #   CMath.sinh(1 + 1i) #=> (0.6349639147847361+1.2984575814159773i)
   def sinh(z)
     begin
       if z.real?
-	sinh!(z)
+        RealMath.sinh(z)
       else
-	Complex(sinh!(z.real) * cos!(z.imag),
-		cosh!(z.real) * sin!(z.imag))
+        Complex(RealMath.sinh(z.real) * RealMath.cos(z.imag),
+                RealMath.cosh(z.real) * RealMath.sin(z.imag))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -233,16 +226,16 @@ module CMath
   end
 
   ##
-  # returns the hyperbolic cosine of +z+, where +z+ is given in radians
+  # Returns the hyperbolic cosine of +z+, where +z+ is given in radians
   #
   #   CMath.cosh(1 + 1i) #=> (0.8337300251311491+0.9888977057628651i)
   def cosh(z)
     begin
       if z.real?
-	cosh!(z)
+        RealMath.cosh(z)
       else
-	Complex(cosh!(z.real) * cos!(z.imag),
-		sinh!(z.real) * sin!(z.imag))
+        Complex(RealMath.cosh(z.real) * RealMath.cos(z.imag),
+                RealMath.sinh(z.real) * RealMath.sin(z.imag))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -250,15 +243,15 @@ module CMath
   end
 
   ##
-  # returns the hyperbolic tangent of +z+, where +z+ is given in radians
+  # Returns the hyperbolic tangent of +z+, where +z+ is given in radians
   #
   #   CMath.tanh(1 + 1i) #=> (1.0839233273386943+0.27175258531951174i)
   def tanh(z)
     begin
       if z.real?
-	tanh!(z)
+        RealMath.tanh(z)
       else
-	sinh(z) / cosh(z)
+        sinh(z) / cosh(z)
       end
     rescue NoMethodError
       handle_no_method_error
@@ -266,15 +259,15 @@ module CMath
   end
 
   ##
-  # returns the arc sine of +z+
+  # Returns the arc sine of +z+
   #
   #   CMath.asin(1 + 1i) #=> (0.6662394324925153+1.0612750619050355i)
   def asin(z)
     begin
       if z.real? and z >= -1 and z <= 1
-	asin!(z)
+        RealMath.asin(z)
       else
-	(-1.0).i * log(1.0.i * z + sqrt(1.0 - z * z))
+        (-1.0).i * log(1.0.i * z + sqrt(1.0 - z * z))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -282,15 +275,15 @@ module CMath
   end
 
   ##
-  # returns the arc cosine of +z+
+  # Returns the arc cosine of +z+
   #
   #   CMath.acos(1 + 1i) #=> (0.9045568943023813-1.0612750619050357i)
   def acos(z)
     begin
       if z.real? and z >= -1 and z <= 1
-	acos!(z)
+        RealMath.acos(z)
       else
-	(-1.0).i * log(z + 1.0.i * sqrt(1.0 - z * z))
+        (-1.0).i * log(z + 1.0.i * sqrt(1.0 - z * z))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -298,15 +291,15 @@ module CMath
   end
 
   ##
-  # returns the arc tangent of +z+
+  # Returns the arc tangent of +z+
   #
   #   CMath.atan(1 + 1i) #=> (1.0172219678978514+0.4023594781085251i)
   def atan(z)
     begin
       if z.real?
-	atan!(z)
+        RealMath.atan(z)
       else
-	1.0.i * log((1.0.i + z) / (1.0.i - z)) / 2.0
+        1.0.i * log((1.0.i + z) / (1.0.i - z)) / 2.0
       end
     rescue NoMethodError
       handle_no_method_error
@@ -321,9 +314,9 @@ module CMath
   def atan2(y,x)
     begin
       if y.real? and x.real?
-	atan2!(y,x)
+        RealMath.atan2(y,x)
       else
-	(-1.0).i * log((x + 1.0.i * y) / sqrt(x * x + y * y))
+        (-1.0).i * log((x + 1.0.i * y) / sqrt(x * x + y * y))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -337,9 +330,9 @@ module CMath
   def asinh(z)
     begin
       if z.real?
-	asinh!(z)
+        RealMath.asinh(z)
       else
-	log(z + sqrt(1.0 + z * z))
+        log(z + sqrt(1.0 + z * z))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -353,9 +346,9 @@ module CMath
   def acosh(z)
     begin
       if z.real? and z >= 1
-	acosh!(z)
+        RealMath.acosh(z)
       else
-	log(z + sqrt(z * z - 1.0))
+        log(z + sqrt(z * z - 1.0))
       end
     rescue NoMethodError
       handle_no_method_error
@@ -369,9 +362,9 @@ module CMath
   def atanh(z)
     begin
       if z.real? and z >= -1 and z <= 1
-	atanh!(z)
+        RealMath.atanh(z)
       else
-	log((1.0 + z) / (1.0 - z)) / 2.0
+        log((1.0 + z) / (1.0 - z)) / 2.0
       end
     rescue NoMethodError
       handle_no_method_error
