@@ -1,32 +1,17 @@
+# frozen_string_literal: true
+
 require 'prettyprint'
 
-module Kernel
-  # returns a pretty printed object as a string.
-  def pretty_inspect
-    PP.pp(self, '')
-  end
-
-  private
-  # prints arguments in pretty form.
-  #
-  # pp returns argument(s).
-  def pp(*objs) # :doc:
-    objs.each {|obj|
-      PP.pp(obj)
-    }
-    objs.size <= 1 ? objs.first : objs
-  end
-  module_function :pp
-end
-
-# == Pretty-printer for Ruby objects.
+##
+# A pretty-printer for Ruby objects.
 #
-# = Which seems better?
+##
+# == What PP Does
 #
-# non-pretty-printed output by #p is:
+# Standard output by #p returns this:
 #   #<PP:0x81fedf0 @genspace=#<Proc:0x81feda0>, @group_queue=#<PrettyPrint::GroupQueue:0x81fed3c @queue=[[#<PrettyPrint::Group:0x81fed78 @breakables=[], @depth=0, @break=false>], []]>, @buffer=[], @newline="\n", @group_stack=[#<PrettyPrint::Group:0x81fed78 @breakables=[], @depth=0, @break=false>], @buffer_width=0, @indent=0, @maxwidth=79, @output_width=2, @output=#<IO:0x8114ee4>>
 #
-# pretty-printed output by #pp is:
+# Pretty-printed output returns this:
 #   #<PP:0x81fedf0
 #    @buffer=[],
 #    @buffer_width=0,
@@ -44,32 +29,42 @@ end
 #    @output=#<IO:0x8114ee4>,
 #    @output_width=2>
 #
-# I like the latter.  If you do too, this library is for you.
-#
-# = Usage
+##
+# == Usage
 #
 #   pp(obj)             #=> obj
+#   pp obj              #=> obj
 #   pp(obj1, obj2, ...) #=> [obj1, obj2, ...]
 #   pp()                #=> nil
 #
-# output +obj(s)+ to +$>+ in pretty printed format.
+# Output <tt>obj(s)</tt> to <tt>$></tt> in pretty printed format.
 #
-# It returns +obj(s)+.
+# It returns <tt>obj(s)</tt>.
 #
-# = Output Customization
-# To define your customized pretty printing function for your classes,
-# redefine a method #pretty_print(+pp+) in the class.
-# It takes an argument +pp+ which is an instance of the class PP.
-# The method should use PP#text, PP#breakable, PP#nest, PP#group and
-# PP#pp to print the object.
+##
+# == Output Customization
 #
-# = Author
-# Tanaka Akira <akr@m17n.org>
+# To define a customized pretty printing function for your classes,
+# redefine method <code>#pretty_print(pp)</code> in the class.
+#
+# <code>#pretty_print</code> takes the +pp+ argument, which is an instance of the PP class.
+# The method uses #text, #breakable, #nest, #group and #pp to print the
+# object.
+#
+##
+# == Pretty-Print JSON
+#
+# To pretty-print JSON refer to JSON#pretty_generate.
+#
+##
+# == Author
+# Tanaka Akira <akr@fsij.org>
+
 class PP < PrettyPrint
   # Outputs +obj+ to +out+ in pretty printed format of
   # +width+ columns in width.
   #
-  # If +out+ is omitted, +$>+ is assumed.
+  # If +out+ is omitted, <code>$></code> is assumed.
   # If +width+ is omitted, 79 is assumed.
   #
   # PP.pp returns +out+.
@@ -106,33 +101,44 @@ class PP < PrettyPrint
   end
 
   module PPMethods
+
+    # Yields to a block
+    # and preserves the previous set of objects being printed.
     def guard_inspect_key
       if Thread.current[:__recursive_key__] == nil
-        Thread.current[:__recursive_key__] = {}.untrust
+        Thread.current[:__recursive_key__] = {}.taint
       end
 
       if Thread.current[:__recursive_key__][:inspect] == nil
-        Thread.current[:__recursive_key__][:inspect] = {}.untrust
+        Thread.current[:__recursive_key__][:inspect] = {}.taint
       end
 
       save = Thread.current[:__recursive_key__][:inspect]
 
       begin
-        Thread.current[:__recursive_key__][:inspect] = {}.untrust
+        Thread.current[:__recursive_key__][:inspect] = {}.taint
         yield
       ensure
         Thread.current[:__recursive_key__][:inspect] = save
       end
     end
 
+    # Check whether the object_id +id+ is in the current buffer of objects
+    # to be pretty printed. Used to break cycles in chains of objects to be
+    # pretty printed.
     def check_inspect_key(id)
       Thread.current[:__recursive_key__] &&
       Thread.current[:__recursive_key__][:inspect] &&
       Thread.current[:__recursive_key__][:inspect].include?(id)
     end
+
+    # Adds the object_id +id+ to the set of objects being pretty printed, so
+    # as to not repeat objects.
     def push_inspect_key(id)
       Thread.current[:__recursive_key__][:inspect][id] = true
     end
+
+    # Removes an object from the set of objects being pretty printed.
     def pop_inspect_key(id)
       Thread.current[:__recursive_key__][:inspect].delete id
     end
@@ -165,18 +171,12 @@ class PP < PrettyPrint
       group(1, '#<' + obj.class.name, '>', &block)
     end
 
-    PointerMask = (1 << ([""].pack("p").size * 8)) - 1
-
-    case Object.new.inspect
-    when /\A\#<Object:0x([0-9a-f]+)>\z/
-      PointerFormat = "%0#{$1.length}x"
-    else
-      PointerFormat = "%x"
-    end
-
+    # A convenience method, like object_group, but also reformats the Object's
+    # object_id.
     def object_address_group(obj, &block)
-      id = PointerFormat % (obj.object_id * 2 & PointerMask)
-      group(1, "\#<#{obj.class}:0x#{id}", '>', &block)
+      str = Kernel.instance_method(:to_s).bind(obj).call
+      str.chomp!('>')
+      group(1, str, '>', &block)
     end
 
     # A convenience method which is same as follows:
@@ -225,6 +225,7 @@ class PP < PrettyPrint
       }
     end
 
+    # A present standard failsafe for pretty printing any given Object
     def pp_object(obj)
       object_address_group(obj) {
         seplist(obj.pretty_print_instance_variables, lambda { text ',' }) {|v|
@@ -240,6 +241,7 @@ class PP < PrettyPrint
       }
     end
 
+    # A pretty print for a Hash
     def pp_hash(obj)
       group(1, '{', '}') {
         seplist(obj, nil, :each_pair) {|k, v|
@@ -258,11 +260,11 @@ class PP < PrettyPrint
 
   include PPMethods
 
-  class SingleLine < PrettyPrint::SingleLine
+  class SingleLine < PrettyPrint::SingleLine # :nodoc:
     include PPMethods
   end
 
-  module ObjectMixin
+  module ObjectMixin # :nodoc:
     # 1. specific pretty_print
     # 2. specific inspect
     # 3. generic pretty_print
@@ -282,7 +284,7 @@ class PP < PrettyPrint
         inspect_method = method_method.call(:inspect)
       rescue NameError
       end
-      if inspect_method && /\(Kernel\)#/ !~ inspect_method.inspect
+      if inspect_method && inspect_method.owner != Kernel
         q.text self.inspect
       elsif !inspect_method && self.respond_to?(:inspect)
         q.text self.inspect
@@ -316,16 +318,16 @@ class PP < PrettyPrint
     # However, doing this requires that every class that #inspect is called on
     # implement #pretty_print, or a RuntimeError will be raised.
     def pretty_print_inspect
-      if /\(PP::ObjectMixin\)#/ =~ Object.instance_method(:method).bind(self).call(:pretty_print).inspect
+      if Object.instance_method(:method).bind(self).call(:pretty_print).owner == PP::ObjectMixin
         raise "pretty_print is not overridden for #{self.class}"
       end
-      PP.singleline_pp(self, '')
+      PP.singleline_pp(self, ''.dup)
     end
   end
 end
 
-class Array
-  def pretty_print(q)
+class Array # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.group(1, '[', ']') {
       q.seplist(self) {|v|
         q.pp v
@@ -333,23 +335,23 @@ class Array
     }
   end
 
-  def pretty_print_cycle(q)
+  def pretty_print_cycle(q) # :nodoc:
     q.text(empty? ? '[]' : '[...]')
   end
 end
 
-class Hash
-  def pretty_print(q)
+class Hash # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.pp_hash self
   end
 
-  def pretty_print_cycle(q)
+  def pretty_print_cycle(q) # :nodoc:
     q.text(empty? ? '{}' : '{...}')
   end
 end
 
-class << ENV
-  def pretty_print(q)
+class << ENV # :nodoc:
+  def pretty_print(q) # :nodoc:
     h = {}
     ENV.keys.sort.each {|k|
       h[k] = ENV[k]
@@ -358,8 +360,8 @@ class << ENV
   end
 end
 
-class Struct
-  def pretty_print(q)
+class Struct # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.group(1, sprintf("#<struct %s", PP.mcall(self, Kernel, :class).name), '>') {
       q.seplist(PP.mcall(self, Struct, :members), lambda { q.text "," }) {|member|
         q.breakable
@@ -373,13 +375,13 @@ class Struct
     }
   end
 
-  def pretty_print_cycle(q)
+  def pretty_print_cycle(q) # :nodoc:
     q.text sprintf("#<struct %s:...>", PP.mcall(self, Kernel, :class).name)
   end
 end
 
-class Range
-  def pretty_print(q)
+class Range # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.pp self.begin
     q.breakable ''
     q.text(self.exclude_end? ? '...' : '..')
@@ -388,9 +390,24 @@ class Range
   end
 end
 
-class File < IO
-  class Stat
-    def pretty_print(q)
+class String # :nodoc:
+  def pretty_print(q) # :nodoc:
+    lines = self.lines
+    if lines.size > 1
+      q.group(0, '', '') do
+        q.seplist(lines, lambda { q.text ' +'; q.breakable }) do |v|
+          q.pp v
+        end
+      end
+    else
+      q.text inspect
+    end
+  end
+end
+
+class File < IO # :nodoc:
+  class Stat # :nodoc:
+    def pretty_print(q) # :nodoc:
       require 'etc.so'
       q.object_group(self) {
         q.breakable
@@ -441,8 +458,10 @@ class File < IO
         q.comma_breakable
         q.group {
           q.text sprintf("rdev=0x%x", self.rdev)
-          q.breakable
-          q.text sprintf('(%d, %d)', self.rdev_major, self.rdev_minor)
+          if self.rdev_major && self.rdev_minor
+            q.breakable
+            q.text sprintf('(%d, %d)', self.rdev_major, self.rdev_minor)
+          end
         }
         q.comma_breakable
         q.text "size="; q.pp self.size; q.comma_breakable
@@ -470,8 +489,8 @@ class File < IO
   end
 end
 
-class MatchData
-  def pretty_print(q)
+class MatchData # :nodoc:
+  def pretty_print(q) # :nodoc:
     nc = []
     self.regexp.named_captures.each {|name, indexes|
       indexes.each {|i| nc[i] = name }
@@ -495,7 +514,7 @@ class MatchData
   end
 end
 
-class Object < BasicObject
+class Object < BasicObject # :nodoc:
   include PP::ObjectMixin
 end
 
@@ -514,3 +533,27 @@ end
     end
   }
 }
+
+module Kernel
+  # Returns a pretty printed object as a string.
+  #
+  # In order to use this method you must first require the PP module:
+  #
+  #   require 'pp'
+  #
+  # See the PP module for more information.
+  def pretty_inspect
+    PP.pp(self, ''.dup)
+  end
+
+  # prints arguments in pretty form.
+  #
+  # pp returns argument(s).
+  def pp(*objs)
+    objs.each {|obj|
+      PP.pp(obj)
+    }
+    objs.size <= 1 ? objs.first : objs
+  end
+  module_function :pp
+end
