@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("2.5.") or
-    raise "ruby lib version (2.5.7) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (2.5.8) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.5/sparc64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "2"
   CONFIG["MINOR"] = "5"
-  CONFIG["TEENY"] = "7"
-  CONFIG["PATCHLEVEL"] = "206"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.5.7/bin/install -c'
+  CONFIG["TEENY"] = "8"
+  CONFIG["PATCHLEVEL"] = "224"
+  CONFIG["INSTALL"] = '/build/tmp/pobj//ruby-2.5.8/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)25"
@@ -146,9 +146,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.5.7/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.5.7/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.5.7/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/build/tmp/pobj//ruby-2.5.8/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/build/tmp/pobj//ruby-2.5.8/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/build/tmp/pobj//ruby-2.5.8/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -197,8 +197,8 @@ module RbConfig
   CONFIG["build_cpu"] = "sparc64"
   CONFIG["build"] = "sparc64-unknown-openbsd6.6"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["RUBY_PROGRAM_VERSION"] = "2.5.7"
-  CONFIG["HAVE_GIT"] = "no"
+  CONFIG["RUBY_PROGRAM_VERSION"] = "2.5.8"
+  CONFIG["HAVE_GIT"] = "yes"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = "$(optflags) $(debugflags) $(warnflags)"
   CONFIG["cppflags"] = ""
